;;;ITEM-T to BOM Entry J. Calvi
;; ACAD/BricsCAD Lewis BOM Export Lisp Routine
;; Copyright J.Calvi 2021
;; Version 1.0 
(defun c:ItemT2BomEntry ( / *error* del des ent idx lst obj ord out sel )
  ;;Error Function for catching errors, breaks etc.
   (defun *error* ( msg )
       (if (= 'file (type des))
           (close des)
       )
       (if (not (wcmatch (strcase msg t) "*break,*cancel*,*exit*"))
           (princ (strcat "\nError: " msg))
       )
       (princ)
   )

   (princ "ITEM-T 2 BOM Entry by John Calvi\n\r")
   (princ "Info: ")
   (EraseBlock "PANEL-2") ;;Remove old blocks
   (EraseBlock "PANEL-3") ;;Remove old blocks
   (EraseBlock "PANEL-4") ;;Remove old blocks
   (RENAttrib "ITEM-T" "DRGNO" "DRG_NO") ;; Alter the Drawing Tag
   (RENAttrib "ITEM-T" "DET" "ITEM") ;; Alter the Detail Tag
   (RENAttrib "ITEM-T" "DESC" "DESCRIPTION") ;; Alter the Description Tag
   (RENAttrib "ITEM-T" "MATERIAL" "REMARKS") ;; Alter the Material Tag
   (command "-rename" "b" "ITEM-T" "BOM Entry") ;; Rename the blocks
   (command "attdia" "1") ;;Turn Attribute Dialog On
   (command "-insert" "BOM Entry=L:/AutoCAD/Acad Sheets/Lewis/BOM Entry.dwg" "0.0,0.0,0.0" "" "" "") ;;Insert the new Block
   (command "._erase" "l" "") ;;Remove it again
   (command "attsync" "n" "BOM Entry") ;;Sync the new block
   
)

(defun RENAttrib ($blk $old $new / blocks bo eo ao)
  ;; Get blocks collection in current drawing
  (setq blocks (vla-get-Blocks (vla-get-ActiveDocument (vlax-get-acad-object))))
  ;; Step through all blocks
  (vlax-for bo blocks
    ;; Step through all entities inside block
    (vlax-for eo bo
      (cond
        ;; If attdef & in target block & old tag
        ((and (= (vla-get-ObjectName eo) "AcDbAttributeDefinition")
              (= (strcase (vla-get-Name bo)) (strcase $blk))
              (= (vla-get-TagString eo) $old)
         ) ;_ end of and
         (vla-put-TagString eo $new) ;Change to new name
        )
       
        ;; If block reference & target block
        ((and (= (vla-get-ObjectName eo) "AcDbBlockReference")
              (= (strcase (vla-get-EffectiveName eo)) (strcase $blk))
         ) ;_ end of and
         ;; Step through all attributes
         (foreach ao (vlax-safearray->list (vlax-variant-value (vla-GetAttributes eo)))
           ;; Check if target attrib
           (if (= (strcase (vla-get-TagString ao)) (strcase $old))
             (vla-put-TagString ao $new) ;Change to new name
           ) ;_ end of if
         ) ;_ end of foreach
        )
      ) ;_ end of cond
    ) ;_ end of vlax-for
  ) ;_ end of vlax-for
) ;_ end of defun 

(defun EraseBlock (blk / ss )
       (if (setq ss (ssget "X" (list '(0 . "INSERT") (cons 2 blk)))) 
		(progn		
			(command "_erase" ss "")
			(command "_purge" "B" blk "N")
		)		
    )
)


(vl-load-com) (princ)
